﻿using System.Net.Http;
using System.Text;
using System.Text.Json;

namespace TestLibrary;

/// <summary>
/// OLLAMA 클라이언트
/// </summary>
public class OllamaClient
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 기본 URL
    /// </summary>
    private readonly string baseURL;

    /// <summary>
    /// 모델명
    /// </summary>
    private readonly string modelName;

    /// <summary>
    /// HTTP 클라이언트
    /// </summary>
    private readonly HttpClient client;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - OllamaClient(baseURL, modelName)

    /// <summary>
    /// 생성자
    /// </summary>
    /// <param name="baseURL">기본 URL</param>
    /// <param name="modelName">모델명</param>
    public OllamaClient(string baseURL = "http://localhost:11434", string modelName = "bnksys/eeve-yanolja-v1:latest")
    {
        this.baseURL = baseURL;
        this.modelName = modelName;

        this.client = new HttpClient();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 질문하기 - Ask(question)

    /// <summary>
    /// 질문하기
    /// </summary>
    /// <param name="question">질문</param>
    /// <returns>답변</returns>
    public string Ask(string question)
    {
        var requestContent = new
        {
            model  = this.modelName,
            prompt = question,
            stream = false,
            raw    = false
        };

        string json = JsonSerializer.Serialize(requestContent);

        StringContent stringContent = new StringContent(json, Encoding.UTF8, "application/json");

        using(HttpResponseMessage httpResponseMessage = client.PostAsync($"{baseURL}/api/generate", stringContent).Result)
        {
            httpResponseMessage.EnsureSuccessStatusCode();

            string respnseJSON = httpResponseMessage.Content.ReadAsStringAsync().Result;
                
            using(JsonDocument jsonElement = JsonDocument.Parse(respnseJSON))
            {
                JsonElement rootJSONElement = jsonElement.RootElement;

                if(rootJSONElement.TryGetProperty("response", out JsonElement responseElement))
                {
                    return responseElement.GetString() ?? string.Empty;
                }

                return string.Empty;
            }
        }
    }

    #endregion
}